/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Url Copy: TransferSrmCopy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <Zsolt.Molnar@cern.ch - www.zsoltmolnar.hu>
 *  Version info: $Id: TransferSrmCopy.h,v 1.9 2009/07/02 15:02:09 molnarzs Exp $
 *  Release: $Name:  $
 */

#ifndef GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERSRMCOPY_H_
#define GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERSRMCOPY_H_

#include <glite/data/transfer/urlcopy/TransferSrmCopyStat.h>
#include <glite/data/transfer/urlcopy/details/TransferService.h>
#include <glite/data/transfer/urlcopy/details/SurlPair.h>

#include <string>
#include <vector>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace urlcopy {

/**
 * The Transfer Srm Copy Class
 */
class TransferSrmCopy : public details::TransferService<TransferSrmCopyStat> {
public:
    /**
     * The Transfer Srm Copy Pointer
     */
    typedef boost::shared_ptr<TransferSrmCopy> Ptr;

    /**
     * The Transfer Srm Copy Arguments
     */
    struct Args : public TransferBase::Args {
    	std::vector<details::SurlPair> files;
    	// Source
        std::string src_endpoint;
        std::string src_version;
        // Destination
        std::string dest_endpoint;
        std::string dest_version;
        // Configuration
        bool        pull_mode;
        // Timeouts
        int         copy_timeout;
        int         refresh_timeout;
        /**
         * Constructor
         */
        Args();
    };

    /**
     * Factory method to start a transfer
     * @param args [IN] the transfer arguments
     * @param prefix [IN]
     * @param blocking [IN]
     * @return pointer to transfer object
     */
    static TransferSrmCopy::Ptr start(
        const TransferSrmCopy::Args&    args,
        const std::string&              prefix = "",
        bool                            blocking = false) /*throw (LogicError, RuntimeError)*/;

    /**
     * Factory method to get an existing transfer object
     * @param id [IN] id of the transfer
     * @return pointer to transfer object
     */
    static TransferSrmCopy::Ptr get(const std::string& id) /*throw (LogicError, RuntimeError)*/;

    /**
     * Destructor
     */
    virtual ~TransferSrmCopy();

    /**
     * Return the exec name used
     */
    virtual const std::string& execName() const throw();

    /**
     * Retrieve transfer status
     * @param check [IN] perform a check that the transfer process service the
     *    request is still active.
     */
    virtual TransferSrmCopyStat status(bool check = true) /*throw(RuntimeError)*/;

    /**
     * Stop the transfer
     */
    virtual void stop() /*throw (RuntimeError)*/;

    /**
     * Abort transfer
     */
    virtual void abort() /* throw() */;

    /**
     * Clean the transfer log files
     */
    virtual void clean() /* throw(LogicError) */;

    /**
     * Continue the transfer after preparation/transfer phase has completed.
     */
    virtual pid_t continueTransfer() /* throw(LogicError) */;

    /**
     * Execute the transfer in current the process
     */
    virtual void exec(bool enable_console = false) /*throw(RuntimeError)*/;

    /**
     * Dump the content of a stat file into standard output
     */
    static void dump(const std::string& file_name) /*throw (LogicError, RuntimeError)*/;

protected:
    /**
     * Constructor
     */
    TransferSrmCopy(const std::string& id);

private:
    /**
     * Default Constructor. Not Implemented
     */
    TransferSrmCopy();
};

} // namespace urlcopy
} // namespace transfer
} // namespace data
} // namespace glite

#endif //GLITE_DATA_TRANSFER_URL_COPY_URL_TRANSFERSRMCOPY_H_
